package twentyonecoders;

import javafx.application.Application;
import javafx.scene.Scene;
import javafx.scene.image.Image;
import javafx.stage.Stage;
import javafx.stage.StageStyle;
import javafx.scene.media.Media;
import javafx.scene.media.MediaPlayer;

import twentyonecoders.GUI.LandingPage;
import twentyonecoders.GUI.Main;
import twentyonecoders.GUI.components.CustomDialog;
import twentyonecoders.logic.*;
import twentyonecoders.utils.LogUtil;

import java.io.File;
import java.util.ArrayList;

/**
 * @author Marco Sciacovelli
 */
public class Controller extends Application {

    private Player player;
    private Player dealer;
    private Deck deck;
    private Player winningPlayer;
    private Scene scene;
    private Stage stage;
    public volatile boolean roundOver;
    public boolean naturalBJ = false;
    private MediaPlayer mp; //needs to be here to avoid being deleted by garbage collector
    public LandingPage landingPage = new LandingPage(this);
    public Main mainPage;
    private final LogUtil LOG = new LogUtil();

    /**
     * returns the player of the game.
     * @return of the game.
     */
    public Player getPlayer() {
        return this.player;
    }

    /**
     * This method sets the player that is playing.
     * @param player to be set.
     */
    public void setPlayer(Player player) {
        this.player = player;
    }

    /**
     * Returns the dealer of the game.
     * @return the dealer.
     */
    public Player getDealer() {
        return dealer;
    }

    /**
     * Returns the deck that gets used in the game.
     * @return the deck.
     */
    public Deck getDeck() {
        return deck;
    }

    public static void main(String[] args) {
        launch();
    }

    /**
     * Changes the scene to the passed parameter.
     * @param scene to be set.
     */
    public void changeScene(Scene scene) {
        this.scene = scene;
        this.stage.setScene(this.scene);
        this.stage.show();
    }

    /**
     * Starts the main components of the game like the gui or the soundtrack.
     * @param stage main container.
     */
    @Override
    public void start(Stage stage) {
        LOG.log(player, "Game started");
        Media hit = new Media(String.valueOf(this.getClass().getResource("/Sounds/Song.mp3")));
        mp = new MediaPlayer(hit);
        mp.setVolume(0.1);
        mp.setAutoPlay(true);
        mp.play();
        mp.setOnEndOfMedia(() -> mp.play());
        this.stage = stage;
        this.deck = new Deck();
        this.deck.shuffle();
        this.dealer = new Player(Config.BANK_NAME, Integer.MAX_VALUE);
        this.roundOver = false;
        this.stage.initStyle(StageStyle.UNDECORATED);
        scene = new Scene(landingPage, Config.SCREEN_WIDTH, Config.SCREEN_HEIGHT);
        this.stage.setScene(scene);
        this.stage.getIcons().add(new Image(String.valueOf(this.getClass().getResource("/Images/icon.png"))));
        this.stage.show();
    }

    /**
     * Reloads the main UI and starts another round.
     */
    public void reloadGame() {
        this.mainPage = new Main(this);
        this.scene = new Scene(mainPage, Config.SCREEN_WIDTH, Config.SCREEN_HEIGHT);
        this.stage.setScene(this.scene);
        this.stage.show();
    }

    /**
     * Adds the passed amount to the player's current bet.
     * @param player that bet.
     * @param amount to bet.
     */
    public void bet(Player player, int amount) {
        if(player.getMoney() - (player.getCurrentBet() + amount) >= 0) {
            player.setCurrentBet(amount);
            LOG.log(player, "has set a bet of " + player.getCurrentBet());
            this.refreshUI();
            playSound(String.valueOf(this.getClass().getResource("/Sounds/chips.wav")));
        }
    }

    /**
     * Checks if the player's hand has a value mayor to 21.
     * @return if the player has busted.
     */
    public boolean playerBusted() {
        return this.player.getHand().getValue() > 21;
    }

    /**
     * Checks the hand of the player and of the dealer, sets the winning player and shows the appropriate dialogs.
     */
    public void checkHand() {
        if(player.getHand().getValue() == 21 || dealer.getHand().getValue() > 21 || (dealer.getHand().getValue() < 21 && player.getHand().getValue() < 21 && player.getHand().getValue() > dealer.getHand().getValue())) {
            winningPlayer = player;
            roundOver = true;
            showDialog( "Player won", "You have won against the dealer and got 2 times your bet.");
        } else if(dealer.getHand().getValue() == 21 || player.getHand().getValue() > 21 || (dealer.getHand().getValue() < 21 && player.getHand().getValue() < 21 && player.getHand().getValue() < dealer.getHand().getValue())) {
            winningPlayer = dealer;
            roundOver = true;
            showDialog( "Dealer won", "The Dealer has won, you have lost your bet.");
        } else if(dealer.getHand().getValue() == player.getHand().getValue()) {
            winningPlayer = null;
            roundOver = true;
            showDialog( "Both have the same value", "You get your bet back.");
        }
    }

    /**
     * Checks if the Player has a natural BlackJack, that is composed by an ace and some other card of the value ten.
     */
    public void checkForNaturalBJ() {
        if(player.getHand().getValue() == 21 && dealer.getHand().getValue() == 21) {
            showDialog("Both have a natural BlackJack","");
        } else if(player.getHand().getValue() == 21 && player.getHand().getCards().size() == 2) {
            winningPlayer = player;
            roundOver = true;
            naturalBJ = true;
            showDialog("You have a natural BlackJack","You have won and get 2.5 times your bet back.");
        }
    }

    /**
     * Shows a custom dialog with the passed details.
     * @param header of the dialog.
     * @param text of the dialog.
     */
    public void showDialog(String header, String text) {
        CustomDialog dialog = new CustomDialog(header, text,"Ok","Exit");
        dialog.display();
        dialog.setConfirmEvent(event -> {
            dealer.setHand(new Hand(new ArrayList<>()));
            player.setHand(new Hand(new ArrayList<>()));

            if(roundOver) {
                if(winningPlayer != null) {
                    if(winningPlayer.equals(player) && naturalBJ) {
                        player.setMoney(player.getMoney() + (int)(player.getCurrentBet() * 2.5));
                        LOG.log(player,"has got a natural BlackJack and won " + player.getCurrentBet() * 2.5 + " credits.");
                    } else if(winningPlayer.equals(player)) {
                        player.setMoney(player.getMoney() + player.getCurrentBet() * 2);
                        LOG.log(player,"has won " + player.getCurrentBet() * 2 + " credits.");
                    }
                } else {
                    player.setMoney(player.getMoney() + player.getCurrentBet());
                    LOG.log(player,"There has been a tie and " + player.getCurrentBet() + " got " + player.getCurrentBet() + "credits back.");
                }
                if(player.getMoney() == 0) {
                    try {
                        throw new BankruptException(player.getName() + " went bankrupt.");
                    } catch (BankruptException e) {
                        LOG.log(player, e.getClass().getName() + " has been thrown with the message: " + e.getMessage());
                        CustomDialog bankruptDialog = new CustomDialog("Bankrupt","You are bankrupt, do you want to start again?","Yes","Exit");
                        bankruptDialog.display();
                        bankruptDialog.setConfirmEvent(cevent -> {
                            bankruptDialog.hideStage();
                            player.setMoney(Config.START_SUM);
                            LOG.log(player," started again.");
                            reloadGame();
                        });
                        bankruptDialog.setDeclineEvent(devent -> {
                            LOG.log(player,"exited the game.");
                            System.exit(0);
                        });
                    }
                }
                dialog.hideStage();
                reloadGame();
            }
        });
        dialog.setDeclineEvent(event -> System.exit(0));
    }

    /**
     * Refreshes the current bet and balance of the player.
     */
    public void refreshUI(){
        mainPage.playerName.setText(player.getName() + ": " + player.getCurrentBet() + "$");
        mainPage.playerMoney.setText(player.getMoney() + "$");
    }

    /**
     * Checks if the player's hand has a value mayor to 21.
     */
    public void checkBusted() {
        if(player.getHand().getValue() > 21) {
            winningPlayer = dealer;
            roundOver = true;
            showDialog( "Dealer won", "The dealer has won, because your hand\nhas a value bigger than 21.");
            LOG.log(player,"Dealer has won since player busted");
        }
    }

    /**
     * Plays an audio, found in the passed address.
     * @param src of the sound.
     */
    public void playSound(String src) {
        Media hit = new Media(src);
        MediaPlayer mediaPlayer = new MediaPlayer(hit);
        mediaPlayer.setVolume(0.1);
        mediaPlayer.play();
    }
}
